@@ -3,7 +3,7 @@
 usage(){
 echo "
 Written by Brian Bushnell
-Last modified September 4, 2018
+Last modified August 8, 2022
 
 Description:  Error corrects reads and/or filters by depth, storing
 kmer counts in a count-min sketch (a Bloom filter variant).
@@ -64,6 +64,8 @@ cells=          Option to set the number of cells manually.  By default this
                 to set this is to ensure deterministic output.
 seed=0          This will change the hash function used.  Useful if running
                 iteratively with a very full table.  -1 uses a random seed.
+lockedincrement=t  Increases counting accuracy for a slight speed penalty.
+                Could have low performance on very low-complexity sequence.
                 
 Depth filtering parameters:
 mincount=0      If positive, reads with kmer counts below mincount will
