@@ -5,6 +5,8 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Random;
 import java.util.concurrent.atomic.AtomicIntegerArray;
+import java.util.concurrent.locks.Lock;
+import java.util.concurrent.locks.ReentrantLock;
 
 import shared.KillSwitch;
 import shared.Primes;
@@ -85,6 +87,12 @@ public final class KCountArray7MTA extends KCountArray {
 //		System.out.println("cells="+cells+", words="+words+", wordsPerArray="+wordsPerArray+", numArrays="+numArrays+", hashes="+hashes);
 		
 		matrix=allocMatrix(numArrays, wordsPerArray);
+		
+		useLocks=(cellBits>1 && hashes>1) && (LOCKED_INCREMENT || !SET_LOCKED_INCREMENT);
+		if(useLocks){
+			locks=new Lock[NUM_LOCKS];
+			for(int i=0; i<NUM_LOCKS; i++){locks[i]=new ReentrantLock();}
+		}else{locks=null;}
 				
 //		matrix=new AtomicIntegerArray[numArrays];
 //		for(int i=0; i<matrix.length; i++){
@@ -266,20 +274,38 @@ public final class KCountArray7MTA extends KCountArray {
 			if(x<prefilterLimit){return;}
 		}
 		
+		if(useLocks){incrementLocked(rawKey);}
+		else{incrementUnlocked(rawKey);}
+	}
+	
+	private void incrementUnlocked(long rawKey){
 		long key2=rawKey;
 		for(int i=0; i<hashes; i++){
 			key2=hash(key2, i);
 			if(verbose){System.err.println("key2="+key2+", value="+readHashed(key2));}
-//			assert(readHashed(key2)==0);
-			
-//			int bnum=(int)(key2&arrayMask);
 			incrementHashedLocal(key2);
-//			assert(read(rawKey)<=min+incr) : "i="+i+", original="+min+", new should be <="+(min+incr)+", new="+read(rawKey)+", max="+maxValue+", key="+rawKey;
-//			assert(readHashed(key2)>=min+incr) : "i="+i+", original="+min+", new should be <="+(min+incr)+", new="+read(rawKey)+", max="+maxValue+", key="+rawKey;
 			key2=Long.rotateRight(key2, hashBits);
 		}
 	}
 	
+	private void incrementLocked(long rawKey){
+		final Lock lock=getLock(rawKey);
+		lock.lock();
+		final int min=read(rawKey);
+		if(min>=maxValue){
+			lock.unlock();
+			return;
+		}
+		long key2=rawKey;
+		for(int i=0; i<hashes; i++){
+			key2=hash(key2, i);
+			if(verbose){System.err.println("key2="+key2+", value="+readHashed(key2));}
+			incrementHashedLocal_ifAtMost(key2, min);
+			key2=Long.rotateRight(key2, hashBits);
+		}
+		lock.unlock();
+	}
+	
 	@Override
 	public final void decrement(final long rawKey){
 		if(verbose){System.err.println("\n*** Decrementing raw key "+rawKey+" ***");}
@@ -315,18 +341,39 @@ public final class KCountArray7MTA extends KCountArray {
 			if(x<prefilterLimit){return x;}
 		}
 		
+		if(useLocks){return incrementAndReturnUnincrementedLocked(rawKey, incr);}
+		else{return incrementAndReturnUnincrementedUnlocked(rawKey, incr);}
+	}
+	
+	private int incrementAndReturnUnincrementedLocked(final long rawKey, final int incr){
+		final Lock lock=getLock(rawKey);
+		lock.lock();
+		final int min=read(rawKey);
+		if(min>=maxValue){
+			lock.unlock();
+			return min;
+		}
+		long key2=rawKey;
+		int value=maxValue;
+		for(int i=0; i<hashes; i++){
+			key2=hash(key2, i);
+			if(verbose){System.err.println("key2="+key2+", value="+readHashed(key2));}
+			int x=incrementHashedLocalAndReturnUnincremented_ifAtMost(key2, incr, min);
+			value=min(value, x);
+			key2=Long.rotateRight(key2, hashBits);
+		}
+		lock.unlock();
+		return value;
+	}
+	
+	private int incrementAndReturnUnincrementedUnlocked(final long rawKey, final int incr){
 		long key2=rawKey;
 		int value=maxValue;
 		for(int i=0; i<hashes; i++){
 			key2=hash(key2, i);
 			if(verbose){System.err.println("key2="+key2+", value="+readHashed(key2));}
-//			assert(readHashed(key2)==0);
-			
-//			int bnum=(int)(key2&arrayMask);
 			int x=incrementHashedLocalAndReturnUnincremented(key2, incr);
 			value=min(value, x);
-//			assert(read(rawKey)<=min+incr) : "i="+i+", original="+min+", new should be <="+(min+incr)+", new="+read(rawKey)+", max="+maxValue+", key="+rawKey;
-//			assert(readHashed(key2)>=min+incr) : "i="+i+", original="+min+", new should be <="+(min+incr)+", new="+read(rawKey)+", max="+maxValue+", key="+rawKey;
 			key2=Long.rotateRight(key2, hashBits);
 		}
 		return value;
@@ -588,6 +635,11 @@ public final class KCountArray7MTA extends KCountArray {
 		}
 	}
 	
+	private final Lock getLock(long rawKey){
+		final Lock lock=locks[(int)((rawKey&Long.MAX_VALUE)%(NUM_LOCKS))];
+		return lock;
+	}
+	
 	private int incrementHashedLocal(long key){
 		final int num=(int)(key&arrayMask);
 		final AtomicIntegerArray array=matrix[num];
@@ -607,6 +659,25 @@ public final class KCountArray7MTA extends KCountArray {
 		return value;
 	}
 	
+	private int incrementHashedLocal_ifAtMost(long key, final int thresh){
+		final int num=(int)(key&arrayMask);
+		final AtomicIntegerArray array=matrix[num];
+		key=(key>>>arrayBits)%(cellMod);
+//		key=(key>>>(arrayBits+1))%(cellMod);
+		int index=(int)(key>>>indexShift);
+		int cellShift=(int)(cellBits*key);
+		int value, word, word2;
+		do{
+			assert(index>=0) : key+", "+cellMod+", "+cellBits+", "+valueMask+", "+arrayMask+", "+index+", "+num;
+			word=array.get(index);
+			value=((word>>>cellShift)&valueMask);
+			if(value>thresh){return value;}//Too high; don't increment
+			value=min(value+1, maxValue);
+			word2=(value<<cellShift)|(word&~((valueMask)<<cellShift));
+		}while(word!=word2 && !array.compareAndSet(index, word, word2));
+		return value;
+	}
+	
 	private int incrementHashedLocalAndReturnUnincremented(long key, int incr){
 		assert(incr>=0);
 		final int num=(int)(key&arrayMask);
@@ -626,6 +697,26 @@ public final class KCountArray7MTA extends KCountArray {
 		return value;
 	}
 	
+	private int incrementHashedLocalAndReturnUnincremented_ifAtMost(long key, int incr, final int thresh){
+		assert(incr>=0);
+		final int num=(int)(key&arrayMask);
+		final AtomicIntegerArray array=matrix[num];
+		key=(key>>>arrayBits)%(cellMod);
+//		key=(key>>>(arrayBits+1))%(cellMod);
+		int index=(int)(key>>>indexShift);
+		int cellShift=(int)(cellBits*key);
+		int value, word, word2;
+		do{
+			word=array.get(index);
+			value=((word>>>cellShift)&valueMask);
+			if(value>thresh){return value;}//Too high; don't increment
+			int value2=min(value+incr, maxValue);
+			word2=(value2<<cellShift)|(word&~((valueMask)<<cellShift));
+		}while(word!=word2 && !array.compareAndSet(index, word, word2));
+//		if(value==1){cellsUsedPersonal.incrementAndGet(num);}
+		return value;
+	}
+	
 	private int decrementHashedLocal(long key){
 		final int num=(int)(key&arrayMask);
 		final AtomicIntegerArray array=matrix[num];
@@ -689,6 +780,10 @@ public final class KCountArray7MTA extends KCountArray {
 	private static final int hashCellMask=hashArrayLength-1;
 	
 	private KCountArray prefilter;
+	
+	private final transient boolean useLocks;
+	private final transient Lock[] locks;
+	private static final transient int NUM_LOCKS=1999;
 
 	private static long counter=0;
 	private static long SEEDMASK=0;
