@@ -16,6 +16,7 @@ import dna.AminoAcid;
 import fileIO.FileFormat;
 import fileIO.ReadWrite;
 import fileIO.TextStreamWriter;
+import shared.Parse;
 import shared.Parser;
 import shared.PreParser;
 import shared.ReadStats;
@@ -29,6 +30,7 @@ import stream.Read;
 import structures.ByteBuilder;
 import structures.ListNum;
 
+
 /**
  * @author Brian Bushnell
  * @date Oct 11, 2012
@@ -121,14 +123,14 @@ public class KmerCoverage {
 				assert(matrixbits<63);
 				cells=1L<<matrixbits;
 			}else if(a.startsWith("cells")){
-				cells=Tools.parseKMG(b);
+				cells=Parse.parseKMG(b);
 			}else if(a.startsWith("precells") || a.startsWith("prefiltercells")){
-				precells=Tools.parseKMG(b);
+				precells=Parse.parseKMG(b);
 				prefilter=prefilter || precells!=0;
 			}else if(a.startsWith("minq")){
 				minq=Byte.parseByte(b);
 			}else if(a.equals("zerobin")){
-				ZERO_BIN=Tools.parseBoolean(b);
+				ZERO_BIN=Parse.parseBoolean(b);
 			}else if(a.startsWith("minmedian")){
 				MIN_MEDIAN=Integer.parseInt(b);
 			}else if(a.startsWith("minaverage")){
@@ -141,39 +143,39 @@ public class KmerCoverage {
 				prehashes=Integer.parseInt(b);
 				prefilter=prefilter || prehashes!=0;
 			}else if(a.equals("prefilter")){
-				prefilter=Tools.parseBoolean(b);
+				prefilter=Parse.parseBoolean(b);
 			}else if(a.startsWith("stepsize") || a.startsWith("buildstepsize")){
 				buildStepsize=Integer.parseInt(b);
 			}else if(a.startsWith("passes") || a.startsWith("buildpasses")){
 				buildpasses=Integer.parseInt(b);
 			}else if(a.equals("printcoverage")){
-				OUTPUT_ATTACHMENT=Tools.parseBoolean(b);
+				OUTPUT_ATTACHMENT=Parse.parseBoolean(b);
 			}else if(a.equals("threads") || a.equals("t")){
 				threads=Integer.parseInt(b);
 			}else if(a.equals("reads") || a.startsWith("maxreads")){
-				maxReads=Tools.parseKMG(b);
+				maxReads=Parse.parseKMG(b);
 			}else if(a.startsWith("tablereads") || a.startsWith("buildreads")){
-				tablereads=Tools.parseKMG(b);
+				tablereads=Parse.parseKMG(b);
 			}else if(a.startsWith("out")){
 				output=b;
 			}else if(a.startsWith("hist")){
 				histFile=b;
 			}else if(a.startsWith("verbose")){
-				verbose=Tools.parseBoolean(b);
+				verbose=Parse.parseBoolean(b);
 			}else if(a.startsWith("datainheader") || a.startsWith("datainname") || a.startsWith("useheader")){
-				USE_HEADER=Tools.parseBoolean(b);
+				USE_HEADER=Parse.parseBoolean(b);
 			}else if(a.equals("ordered") || a.equals("ord")){
-				ordered=Tools.parseBoolean(b);
+				ordered=Parse.parseBoolean(b);
 			}else if(a.equals("append") || a.equals("app")){
-				append=ReadStats.append=Tools.parseBoolean(b);
+				append=ReadStats.append=Parse.parseBoolean(b);
 			}else if(a.equals("overwrite") || a.equals("ow")){
-				overwrite=Tools.parseBoolean(b);
+				overwrite=Parse.parseBoolean(b);
 			}else if(a.equals("auto") || a.equals("automatic")){
-				auto=Tools.parseBoolean(b);
+				auto=Parse.parseBoolean(b);
 			}else if(a.equals("samplewhenreadingtable") || a.equals("sampleoutput")){
-				DONT_SAMPLE_OUTPUT=!Tools.parseBoolean(b);
+				DONT_SAMPLE_OUTPUT=!Parse.parseBoolean(b);
 			}else if(a.equals("dontsamplewhenreadingtable") || a.equals("dontsampleoutput")){
-				DONT_SAMPLE_OUTPUT=Tools.parseBoolean(b);
+				DONT_SAMPLE_OUTPUT=Parse.parseBoolean(b);
 			}else if(a.startsWith("kmersample")){
 				kmersamplerate=Integer.parseInt(b);
 //				KmerCountAbstract.kmersamplerate=kmersamplerate;
@@ -181,13 +183,13 @@ public class KmerCoverage {
 				readsamplerate=Integer.parseInt(b);
 //				KmerCountAbstract.readsamplerate=readsamplerate;
 			}else if(a.startsWith("canonical")){
-				CANONICAL=KmerCountAbstract.CANONICAL=Tools.parseBoolean(b);
+				CANONICAL=KmerCountAbstract.CANONICAL=Parse.parseBoolean(b);
 			}else if(a.startsWith("fixspikes")){
-				FIX_SPIKES=Tools.parseBoolean(b);
+				FIX_SPIKES=Parse.parseBoolean(b);
 			}else if(a.equals("printzerocoverage") || a.equals("pzc")){
-				PRINT_ZERO_COVERAGE=Tools.parseBoolean(b);
+				PRINT_ZERO_COVERAGE=Parse.parseBoolean(b);
 			}else if(a.equals("removeduplicatekmers") || a.equals("rdk")){
-				KmerCountAbstract.KEEP_DUPLICATE_KMERS=!Tools.parseBoolean(b);
+				KmerCountAbstract.KEEP_DUPLICATE_KMERS=!Parse.parseBoolean(b);
 			}else if(a.startsWith("extra")){
 				if(b!=null && !b.equalsIgnoreCase("null")){
 					if(new File(b).exists()){
@@ -225,9 +227,9 @@ public class KmerCoverage {
 			
 			final int maxCount=(int)(cbits>16 ? Integer.MAX_VALUE : (1L<<cbits)-1);
 			assert(maxCount>0);
-			HIST_LEN_PRINT=Tools.max(1, Tools.min(HIST_LEN_PRINT, maxCount));
+			HIST_LEN_PRINT=Tools.max(1, Tools.min(HIST_LEN_PRINT, maxCount+1));
 			assert(HIST_LEN_PRINT<=Integer.MAX_VALUE) : HIST_LEN_PRINT+", "+Integer.MAX_VALUE;
-			HIST_LEN=(int)Tools.min(maxCount, Tools.max(HIST_LEN_PRINT, HIST_LEN));
+			HIST_LEN=(int)Tools.min(maxCount+1, Tools.max(HIST_LEN_PRINT, HIST_LEN));
 			
 			histogram_total=new long[HIST_LEN];
 		}
