@@ -6,6 +6,7 @@ import java.util.Arrays;
 import java.util.HashSet;
 
 import align2.QualityTools;
+import bloom.KCountArray;
 import cardinality.CardinalityTracker;
 import cardinality.LogLog16;
 import cardinality.LogLog2;
@@ -736,6 +737,16 @@ public class Parser {
 			TaxTree.protFull=true;
 		}
 		
+		else if(a.equalsIgnoreCase("lockedincrement")){
+			if("auto".equalsIgnoreCase(b)){
+				KCountArray.LOCKED_INCREMENT=true;
+				KCountArray.SET_LOCKED_INCREMENT=false;
+			}else{
+				KCountArray.LOCKED_INCREMENT=Parse.parseBoolean(b);
+				KCountArray.SET_LOCKED_INCREMENT=true;
+			}
+		}
+		
 		else{
 			return false;
 		}
@@ -1418,6 +1429,10 @@ public class Parser {
 	private static byte qout=-1;
 	private static boolean parsedQuality=false;
 	
+	public static void setQuality(int x){
+		qin=(byte)x;
+		parsedQuality=(x>-1);
+	}
 	public static void processQuality(){
 //		assert(parsedQuality);
 		if(!parsedQuality){return;}
