@@ -277,6 +277,8 @@ public final class TrimRead implements Serializable {
 		
 		if(r.match!=null){
 			return trimReadWithMatch(r, r.samline, leftTrimAmount, rightTrimAmount, minResultingLength, Integer.MAX_VALUE, trimClip);
+		}else if(r.samline!=null){
+			return trimReadWithMatchFast(r, r.samline, leftTrimAmount, rightTrimAmount, minResultingLength);
 		}
 		
 		final byte[] bases=r.bases, qual=r.quality;
@@ -289,6 +291,7 @@ public final class TrimRead implements Serializable {
 		}
 		
 		final int total=leftTrimAmount+rightTrimAmount;
+//		System.err.println("D: L="+leftTrimAmount+", R="+rightTrimAmount+", len="+r.length()+", tot="+total);
 		if(total>0){
 			r.bases=KillSwitch.copyOfRange(bases, leftTrimAmount, len-rightTrimAmount);
 			r.quality=(leftTrimAmount+rightTrimAmount>=qlen ? null : KillSwitch.copyOfRange(qual, leftTrimAmount, qlen-rightTrimAmount));
@@ -577,44 +580,51 @@ public final class TrimRead implements Serializable {
 //			}
 //		}
 	}
-	
-	/** Special case of 100% match */
+
+	/** Special case of 100% match, or no match string */
 	public static int trimReadWithMatchFast(final Read r, final SamLine sl, final int leftTrimAmount, final int rightTrimAmount, final int minFinalLength){
-		assert(r.match!=null);
+		assert(r.match!=null || sl!=null);
 		if(r.bases==null){return 0;}
 		if(leftTrimAmount<1 && rightTrimAmount<1){return 0;}
 		if(leftTrimAmount+rightTrimAmount>=r.length()){return -leftTrimAmount-rightTrimAmount;}
 		
 		final boolean shortmatch=r.shortmatch();
-		final byte[] old=r.match;
+		final byte[] oldMatch=r.match;
 		r.match=null;
+		r.samline=null;
 		final int trimmed;
-		if(sl.strand()==Shared.MINUS){
+//		System.err.println(rightTrimAmount+", "+leftTrimAmount);
+		if(sl!=null && sl.strand()==Shared.MINUS){
 			trimmed=trimByAmount(r, rightTrimAmount, leftTrimAmount, minFinalLength, false);
 		}else{
 			trimmed=trimByAmount(r, leftTrimAmount, rightTrimAmount, minFinalLength, false);
 		}
+		r.samline=sl;
 		if(trimmed<1){
-			r.match=old;
+			r.match=oldMatch;
 			return 0;
 		}
 		
-		ByteBuilder bb=new ByteBuilder();
 		final int len=r.length();
-		if(shortmatch){
-			bb.append((byte)'m');
-			if(len>1){bb.append(len);}
-		}else{
-			for(int i=0; i<len; i++){bb.append((byte)'m');}
+		ByteBuilder bb=new ByteBuilder();
+		if(oldMatch!=null){
+			if(shortmatch){
+				bb.append((byte)'m');
+				if(len>1){bb.append(len);}
+			}else{
+				for(int i=0; i<len; i++){bb.append((byte)'m');}
+			}
+			r.match=bb.toBytes();
+			bb.clear();
 		}
-		r.match=bb.toBytes();
-		bb.clear();
 		
 		if(sl!=null){
 			sl.pos+=leftTrimAmount;
-			if(sl.cigar!=null){
+			if(sl.cigar!=null && sl.cigar.length()>0){
+				char c=sl.cigar.charAt(sl.cigar.length()-1);
+				assert(c=='M' || c=='=') : c+"; "+sl.cigar+"\n"+sl;
 				bb.append(len);
-				bb.append(SamLine.VERSION>1.3 ? '=' : 'm');
+				bb.append(SamLine.VERSION>1.3 ? '=' : 'M');
 				sl.cigar=bb.toString();
 			}
 			sl.seq=r.bases;
@@ -631,11 +641,44 @@ public final class TrimRead implements Serializable {
 		}
 		return trimmed;
 	}
+	
+//	//Should be unneeded, just use the above function
+//	public static int trimReadWithoutMatch(final Read r, final SamLine sl, final int leftTrimAmount, final int rightTrimAmount, final int minFinalLength){
+//		if(r.bases==null){return 0;}
+//		if(leftTrimAmount<1 && rightTrimAmount<1){return 0;}
+//		if(leftTrimAmount+rightTrimAmount>=r.length()){return -leftTrimAmount-rightTrimAmount;}
+//		
+//		assert(r.match==null);
+//		final int trimmed;
+//		if(sl!=null && sl.strand()==Shared.MINUS){
+//			trimmed=trimByAmount(r, rightTrimAmount, leftTrimAmount, minFinalLength, false);
+//		}else{
+//			trimmed=trimByAmount(r, leftTrimAmount, rightTrimAmount, minFinalLength, false);
+//		}
+//		if(trimmed<1){return 0;}
+//		
+//		if(sl!=null){
+//			sl.pos+=leftTrimAmount;
+//			assert(sl.cigar==null);
+//			sl.seq=r.bases;
+//			sl.qual=r.quality;
+//			if(trimmed>0 && sl.optional!=null && sl.optional.size()>0){
+//				ArrayList<String> list=new ArrayList<String>(2);
+//				for(int i=0; i<sl.optional.size(); i++){
+//					String s=sl.optional.get(i);
+//					if(s.startsWith("PG:") || s.startsWith("RG:") || s.startsWith("X") || s.startsWith("Y") || s.startsWith("Z")){list.add(s);} //Only keep safe flags.
+//				}
+//				sl.optional.clear();
+//				sl.optional.addAll(list);
+//			}
+//		}
+//		return trimmed;
+//	}
 
 	//TODO: This is slow
-	//TODO: Note, returns a negative number if the whole read is subbosed to be trimmed
+	//TODO: Note, returns a negative number if the whole read is supposed to be trimmed
 	public static int trimReadWithMatch(final Read r, final SamLine sl, 
-			int leftTrimAmount, int rightTrimAmount, int minFinalLength, int scafLen, boolean trimClip){
+			int leftTrimAmount, int rightTrimAmount, int minFinalLength, long scafLen, boolean trimClip){
 		if(r.bases==null || (leftTrimAmount<1 && rightTrimAmount<1 && !trimClip)){return 0;}
 		if(!r.containsNonM() && !trimClip){
 			return trimReadWithMatchFast(r, sl, leftTrimAmount, rightTrimAmount, minFinalLength);
@@ -681,7 +724,7 @@ public final class TrimRead implements Serializable {
 		if(leftTrimAmount<1 && rightTrimAmount<1){return 0;}
 		if(leftTrimAmount+rightTrimAmount>=r.length()){return -leftTrimAmount-rightTrimAmount;}
 		
-		final int oldPos=sl.pos;
+		final int oldPos=(sl==null ? 1 : sl.pos);
 		
 		r.toLongMatchString(false);
 		byte[] match=r.match;
@@ -768,13 +811,13 @@ public final class TrimRead implements Serializable {
 //		System.err.println("leftTrim="+leftTrim+", rightTrim="+rightTrim);
 		
 		if(leftTrim+rightTrim>=r.length()){
-			sl.pos=oldPos;
+			if(sl!=null){sl.pos=oldPos;}
 			return -leftTrim-rightTrim;
 		}
 		
 //		System.err.println("T: "+sl);
 		r.match=null;
-		if(sl.strand()==Shared.MINUS){
+		if(sl!=null && sl.strand()==Shared.MINUS){
 			int temp=leftTrim;
 			leftTrim=rightTrim;
 			rightTrim=temp;
