@@ -658,7 +658,7 @@ public class SamLine implements Serializable {
 	/*----------------             Cigar            ----------------*/
 	/*--------------------------------------------------------------*/
 	
-	public static String toCigar13(byte[] match, int readStart, int readStop, int reflen, byte[] bases){
+	public static String toCigar13(byte[] match, int readStart, int readStop, long reflen, byte[] bases){
 		if(match==null || readStart==readStop){return null;}
 		ByteBuilder sb=new ByteBuilder(8);
 		int count=0;
@@ -760,7 +760,7 @@ public class SamLine implements Serializable {
 	}
 	
 	
-	public static String toCigar14(byte[] match, int readStart, int readStop, int reflen, byte[] bases){
+	public static String toCigar14(byte[] match, int readStart, int readStop, long reflen, byte[] bases){
 //		assert(false) : readStart+", "+readStop+", "+reflen;
 		if(match==null || readStart==readStop){return null;}
 		ByteBuilder sb=new ByteBuilder(8);
