@@ -10,6 +10,7 @@ import fileIO.FileFormat;
 import fileIO.ReadWrite;
 import fileIO.TextFile;
 import fileIO.TextStreamWriter;
+import shared.Parse;
 import shared.Parser;
 import shared.PreParser;
 import shared.ReadStats;
@@ -86,7 +87,7 @@ public class PrintTaxonomy {
 			}else if(parser.parse(arg, a, b)){//Parse standard flags in the parser
 				//do nothing
 			}else if(a.equals("verbose")){
-				verbose=Tools.parseBoolean(b);
+				verbose=Parse.parseBoolean(b);
 			}else if(a.equals("table") || a.equals("gi") || a.equals("gitable")){
 				giTableFile=b;
 			}else if(a.equals("accession")){
@@ -100,15 +101,15 @@ public class PrintTaxonomy {
 			}else if(a.equals("maxlevel")){
 				maxLevel=TaxTree.parseLevel(b);
 			}else if(a.equals("printname")){
-				printName=Tools.parseBoolean(b);
+				printName=Parse.parseBoolean(b);
 			}else if(a.equals("reverse")){
-				reverseOrder=Tools.parseBoolean(b);
+				reverseOrder=Parse.parseBoolean(b);
 			}else if(a.equals("silva")){
-				TaxTree.SILVA_MODE=Tools.parseBoolean(b);
+				TaxTree.SILVA_MODE=Parse.parseBoolean(b);
 			}else if(a.equals("unite")){
-				TaxTree.UNITE_MODE=Tools.parseBoolean(b);
+				TaxTree.UNITE_MODE=Parse.parseBoolean(b);
 			}else if(a.equals("simple")){
-				skipNonCanonical=Tools.parseBoolean(b);
+				skipNonCanonical=Parse.parseBoolean(b);
 			}else if(a.equals("column")){
 				keyColumn=Integer.parseInt(b);
 			}else if(b!=null && (a.equals("name") || a.equals("names") || a.equals("id") || a.equals("ids"))){
